from silence.settings import settings
from silence.cli.template_downloader import download_from_github

def handle(args):
    args = vars(args)  # Convert the Namespace object into a dict

    if all(not args[k] for k in ("blank", "url", "template")):
        # No extra args provided, use the default template
        args["template"] = settings.DEFAULT_TEMPLATE_NAME
    elif args["blank"]:
        args["template"] = "blank"

    template = args["template"]
    project_name = args["name"]

    if template:
        repo_url = f"https://github.com/{settings.GITHUB_TEMPLATES_OWNER}/silence-template-{template}"
    else:
        # We have to download a repo from a URL
        repo_url = args["url"]

    download_from_github(project_name, repo_url)
    extra_text = f"using the template '{template}'" if template else "from the provided repo"
    print(f'The Silence project "{project_name}" has been created {extra_text}.')


import logging

from silence.settings import settings
from silence.logging.default_formatter import DefaultFormatter

logger = logging.getLogger("silence")
log_lvl = logging.DEBUG if settings.DEBUG_ENABLED else logging.INFO
logger.setLevel(log_lvl)

ch = logging.StreamHandler()
ch.setLevel(log_lvl)
ch.setFormatter(DefaultFormatter())

logger.addHandler(ch)



import pymysql

from silence.settings import settings

###############################################################################
# The connector fetches the relevant configuration parameters
# and uses them to build a connection to the database.
###############################################################################

def get_conn():
    return pymysql.connect(
                host=settings.DB_CONN["host"],
                port=settings.DB_CONN["port"],
                user=settings.DB_CONN["username"],
                password=settings.DB_CONN["password"],
                database=settings.DB_CONN["database"],
            )



###############################################################################
# Default settings, which can be overriden by each individual project.
###############################################################################

# Shows debug messages while Silence is running
DEBUG_ENABLED = False

# The address and port in which the API and the web server will be deployed
LISTEN_ADDRESS = "127.0.0.1"  # Listen only on localhost by default
HTTP_PORT = 8080

# The sequence of SQL scripts located in the sql/ folder that must
# be executed when the 'silence createdb' command is issued
SQL_SCRIPTS = []

# The URL prefix for all API endpoints
API_PREFIX = "/api"

# Enables or disables the API and the web server separately
RUN_API = True
RUN_WEB = True

# Database connection details
DB_CONN = {
    "host": "127.0.0.1",
    "port": 3306,
    "username": "default_username",
    "password": "default_password",
    "database": "default_database",
}

# Table and fields that are used for both login and register
USER_AUTH_DATA = {
    "table": "users",
    "identifier": "username",
    "password": "password",
}

# Enables or disables the /login, /register and summary (/base) endpoints separately
ENABLE_LOGIN = True
ENABLE_REGISTER = True
ENABLE_SUMMARY = True

# Enables or disables the listing of the loaded endpoints on startup
SHOW_ENDPOINT_LIST = True

# Enables or disables colors in the console output
COLORED_OUTPUT = True

# Controls whether DECIMAL types are converted to strings when serializing them to JSON
# By default the are converted into floats, which keep them as numeric values, but it may
# cause inaccuracies for certain values that cannot be stored exactly in a float
DECIMALS_AS_STRINGS = False

# Sets the max-age value (seconds) in the Cache-Control HTTP header for static files
# Defaults to 0 (effectively disabling caching by the browser) for development purposes
HTTP_CACHE_TIME = 0

# Maximum validity time of a session token in seconds
MAX_TOKEN_AGE = 86400

# Default settings for creating a new project
DEFAULT_TEMPLATE_NAME = "employees"
GITHUB_TEMPLATES_OWNER = "IISSI-US"

# Enables or disables checking for new Silence updates when running 'silence run'
CHECK_FOR_UPDATES = True



SELECT = "select"
INSERT = "insert"
UPDATE = "update"
DELETE = "delete"
OTHER = "other"

# Returns the SQL operation for a given string
def get_sql_op(sql):
    first_token = sql.strip().split(" ")[0].lower()

    known_ops = {
        "select": SELECT,
        "insert": INSERT,
        "update": UPDATE,
        "delete": DELETE
    }

    return known_ops.get(first_token, OTHER)


import importlib
import pprint
import sys
import os

from silence.settings import default_settings

###############################################################################
# A Settings class that contains the default settings as attributes,
# overriding them with the user provided settings when necessary.
###############################################################################

class Settings:
    def __init__(self):
        self.setting_keys = set()

        # Load the default settings
        for setting in dir(default_settings):
            if setting.isupper():
                self.setting_keys.add(setting)
                setattr(self, setting, getattr(default_settings, setting))

        # Override the default settings with the user-defined ones
        try:
            sys.path.append(os.getcwd())
            user_settings = importlib.import_module("settings")

            for setting in dir(user_settings):
                if setting.isupper():
                    self.setting_keys.add(setting)
                    setattr(self, setting, getattr(user_settings, setting))

            # Ensure that the user has provided a SECRET_KEY
            if not hasattr(self, "SECRET_KEY"):
                print("You need to provide a SECRET_KEY in your settings.py file.")
                print("If you have git clone'd this project, use 'silence new' instead,")
                print("and it will automatically generate a SECRET_KEY for you.")
                sys.exit(1)

        except ModuleNotFoundError:
            if "run" in sys.argv or "createdb" in sys.argv:
                print("Can't find the settings.py file! Are you running this command from the project's root folder?")
                sys.exit(1)

    def __str__(self):
        return pprint.pformat({setting: getattr(self, setting) for setting in self.setting_keys})
        
settings = Settings()



import importlib

from os import listdir
from os.path import splitext

from silence.server import manager, default_endpoints
from silence.settings import settings
from silence.logging.default_logger import logger

###############################################################################
# Look for .py files inside the project's "/api" folder
# and force them to run, activating the @endpoint decorators
###############################################################################

def load_user_endpoints():
    logger.debug("Looking for custom endpoints...")

    # Load every .py file inside the api/ folder

    for folder in ("api", "endpoints"):
        try:
            pyfiles = [f for f in listdir(folder) if f.endswith(".py")]
        except FileNotFoundError:
            continue

        if folder == "api" and pyfiles:
            logger.warning("Please rename the folder that contains your endpoints to 'endpoints/' instead of 'api/'")
            logger.warning("Support for the 'api/' folder will be dropped in the future.")

        for pyfile in pyfiles:
            module_name = folder + "." + splitext(pyfile)[0]
            logger.debug(f"Found endpoint file: {module_name}")

            try:
                importlib.import_module(module_name)
            except ImportError:
                raise RuntimeError(f"Could not load the API file {module_name}")
    
###############################################################################
# Register the Silence-provided endpoints
###############################################################################
def load_default_endpoints():
    route_prefix = settings.API_PREFIX
    if route_prefix.endswith("/"):
        route_prefix = route_prefix[:-1]

    if settings.ENABLE_SUMMARY:
        manager.API_SUMMARY.register_endpoint({
                "route": route_prefix,
                "method": "GET",
                "desc": "Returns the data regarding the API endpoints",
        })
        manager.APP.add_url_rule(route_prefix, "APItreeHELP", default_endpoints.show_api_endpoints, methods=["GET"])

    if settings.ENABLE_LOGIN:
        login_route = f"{route_prefix}/login"
        manager.API_SUMMARY.register_endpoint({
            "route": login_route,
            "method": "POST",
            "desc": "Starts a new session, returning a session token and the user data if the login is successful",
        })
        manager.APP.add_url_rule(login_route, "login", default_endpoints.login, methods=["POST"])

    if settings.ENABLE_REGISTER:
        register_route = f"{route_prefix}/register"
        manager.API_SUMMARY.register_endpoint({
            "route": register_route,
            "method": "POST",
            "desc": "Creates a new user, returning a session token and the user data if the register is successful",
        })
        manager.APP.add_url_rule(register_route, "register", default_endpoints.register, methods=["POST"])



import re

from silence.logging.default_logger import logger
from silence.settings import settings

###############################################################################
# Utility class for printing the list of loaded endpoints
###############################################################################
class APISummary:
    def __init__(self):
        self.endpoints = []

    def register_endpoint(self, endpoint_data):
        self.endpoints.append(endpoint_data)

    def get_endpoint_list(self):
        return self.endpoints

    def print_endpoints(self):
        # Show a list of endpoints, sorted alphabetically
        if not self.endpoints:
            logger.info("No endpoints loaded.")
            return

        unique_endpoints = {}
        for endpoint in self.endpoints:
            route = endpoint["route"]
            unique_endpoints[route] = unique_endpoints.get(route, []) + [endpoint["method"]]

        unique_endpoints = list(unique_endpoints.items())
        unique_endpoints.sort(key=lambda x: x[0])

        host = settings.LISTEN_ADDRESS
        port = settings.HTTP_PORT
        base = f"http://{host}:{port}"

        logger.info("Endpoints loaded:")

        for route, method_list in unique_endpoints:
            # Force the GET-POST-PUT-DELETE order
            methods = "/".join([m for m in ("GET", "POST", "PUT", "DELETE") if m in method_list])
            # Replace $param with <param>
            route = re.sub(r"\$(\w+)", r"<\1>", route)

            logger.info(f"\t· {base}{route} ({methods})")
    


import os
import re

from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

# Borrowed from urllib3 (which is in turn borrowed from SQLAlchemy)
base_path = os.path.dirname(__file__)
ver_path = os.path.join(base_path, "silence", "__init__.py")

with open(ver_path, "r") as f:
    version = re.compile(r""".*__version__ = ["'](.*?)['"]""", re.S).match(f.read()).group(1)

setup(
    name="Silence",
    version=version,
    description="An educational API+Web framework.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/IISSI-US/Silence",
    author="Agustín Borrego",
    author_email="borrego@us.es",
    license="MIT",
    packages=find_packages(),
    entry_points={
        "console_scripts": [
            "silence = silence.cli.manager:run_from_command_line",
        ],
    },
    install_requires=[
        "colorama~=0.4.3",
        "cryptography~=3.4.6",
        "Flask-Cors==3.0.8",
        "Flask==1.1.2",
        "itsdangerous~=1.1.0",
        "PyMySQL~=0.9.3",
        "pypika==0.37.7",
        "requests~=2.24.0",
        "Werkzeug~=1.0.1",
    ],
    python_requires="~=3.6",
    zip_safe=False
)